#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include "cgic.h"

/*
 * @author	BlackLight	blacklight86@gmail.com
 * 
 * You can find full documentation about the functions and examples of use in the header
 * file "cgic.h" or in the documentation provided with the package
 */

void set_cookie (char *name, char *content)  { printf ("Set-Cookie: %s=%s\n",name,content); }
void location (char *url)  { printf ("Location: %s\n",url); }
void http_header(char *head)  { printf ("%s\n",head); }

void expires (struct tm *ex)  {
	char timestamp[64];

	strftime (timestamp,sizeof(timestamp),"%a, %d %b %Y %H:%M:%S GMT",ex);
	printf ("Expires: %s\n",timestamp);
}

void last_modified (struct tm *last)  {
	char timestamp[64];

	strftime (timestamp,sizeof(timestamp),"%a, %d %b %Y %H:%M:%S GMT",last);
	printf ("Last-Modified: %s\n",last_modified);
}

void html_head()  {
	printf ("Content-Type: text/html\n\n"
		   "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n"
		   "<!HTML header automatically generated by CGI C library by BlackLight>\n");
}

char* server_name()  { return getenv("SERVER_NAME"); }
char* server_software()  { return getenv("SERVER_SOFTWARE"); }
char* request_method()   { return getenv("REQUEST_METHOD");  }
char* self()  { return getenv("SCRIPT_NAME"); }
char* remote_addr()  { return getenv("REMOTE_ADDR"); }

char* remote_host()  {
	if (!getenv("REMOTE_HOST"))
		return remote_addr();
	else
		return getenv("REMOTE_HOST");
}

char* user_agent()  { return getenv("HTTP_USER_AGENT"); }

void set_server_name (char *name)  { setenv ("SERVER_NAME",name,1); }
void set_server_software (char *soft)  { setenv ("SERVER_SOFTWARE",soft,1); }

void unencode(char *src, char *dest, int len)  {
	int i,code;

	for (i=0; i<len; i++, src++, dest++)  {
		if (*src=='+')  *dest=' ';
		else if (*src=='%')  {
			if (sscanf(src+1, "%2x", &code) != 1) code='?';
			*dest = (char) code;
			src += 2;
		}

		else  *dest=*src;
	}

	dest[len]=0;
}

char* get_query()   {
	char *query;

	if (!(query=getenv("QUERY_STRING")))
		return NULL;
	else
		return query;
}

char* post_query()  {
	char *query;
	int len;

	if (!getenv("CONTENT_LENGTH") || sscanf ((char*) getenv("CONTENT_LENGTH"),"%ld",&len) != 1)
		return NULL;

	query = (char*) malloc(++len*sizeof(char));
	fgets(query, len, stdin);
	return query;
}

char* get_field(char *query, char *field)  {
	int i,j,len,pos;
	char *tmp,*input;

	len = strlen(query)+1;

	tmp   = (char*) malloc( (strlen(field)+1)*sizeof(char) );
	input = (char*) malloc(len*sizeof(char));

	sprintf (tmp, "%s=", field);

	if (!strstr(query,tmp))
		return NULL;

	pos = ((int) strstr(query,tmp) - (int) query) + (strlen(field)+1);

	for (i=pos; ; i++)  {
		if (!query[i] || query[i]=='&') break;
	}

	for (j=pos; j<i; j++)
		input[j-pos]=query[j];
	input[j-pos]=0;

	unencode(input,input,len);
	
	return input;
}

char* parse_input (char *input)  {
	int i,j,len;
	char *buff;

	len=strlen(input);

	for (i=0; i<len; i++)  {
		if (input[i]=='<' || input[i]=='>')
			len+=3;
		if (input[i]=='\'' || input[i]=='\"')
			len+=1;
	}

	if (len==strlen(input))
		return input;

	buff = (char*) malloc(len*sizeof(char));
	
	for (i=0, j=0; i<strlen(input); i++, j++)  {
		if (input[i]=='<')  {
			buff[j++]='&';
			buff[j++]='l';
			buff[j++]='t';
			buff[j]=';';
		}

		else if (input[i]=='>')  {
			buff[j++]='&';
			buff[j++]='g';
			buff[j++]='t';
			buff[j]=';';
		}

		else if (input[i]=='\'')  {
			buff[j++]='\\';
			buff[j]='\'';
		}

		else if (input[i]=='\"')  {
			buff[j++]='\\';
			buff[j]='\"';
		} else buff[j]=input[i];
	}

	buff[j]=0;
	return buff;
}

char* cookie (char *cookie)  {
	int i,j,pos,len;
	char *cookie_string;
	char *tmp,*input;

	if (!(cookie_string=getenv("HTTP_COOKIE")))
		return NULL;

	len = strlen(cookie_string)+1;
	tmp   = (char*) malloc( (strlen(cookie)+1)*sizeof(char) );
	input = (char*) malloc(len*sizeof(char));
	sprintf (tmp,"%s=",cookie);

	if (!strstr(cookie_string,tmp))
		return NULL;

	pos = ((int) strstr(cookie_string,tmp) - (int) cookie_string) + (strlen(cookie)+1);

	for (i=pos; ; i++)  {
		if (!cookie_string[i] || cookie_string[i]==';' || cookie_string[i]==' ') break;
	}

	for (j=pos; j<i; j++)
		input[j-pos]=cookie_string[j];
	input[j-pos]=0;

	return input;
}

